package simulation;

import java.util.Iterator;
import java.util.List;

import model.Play;
import theater.Actor;
import theater_intern.IStage;

public class RunSimulator {

	List<Actor> startedActors;
	Object syncObject = new Object();
	IStage newStage;

	public RunSimulator() {
		this.startedActors = null;
		this.newStage = null;
	}

	public void startSimulation() {
		Play play = Play.getPlay();
		synchronized (this.syncObject) {
			this.startedActors = play.getActiveStage().getActors();
			for (Actor actor : this.startedActors) {
				actor.start();
			}
		}
	}

	public void stopSimulation() {
		synchronized (this.syncObject) {
			for (Actor actor : this.startedActors) {
				if (actor != Thread.currentThread()) {
					actor.stop();
				}
			}
		}
	}

	public void suspendSimulation() {
		new Thread() { // damit ein suspend aufrufender Thread auch suspendiert
			// wird
			public void run() {
				synchronized (RunSimulator.this.syncObject) {
					for (Actor actor : RunSimulator.this.startedActors) {
						actor.suspend();
					}
				}
			}
		}.start();
	}

	public void resumeSimulation() {
		synchronized (this.syncObject) {
			for (Actor actor : this.startedActors) {
				actor.resume();
			}
		}
	}

	public void addActor(Actor actor) {
		synchronized (this.syncObject) {
			actor.start();
			if (Play.getPlay().getActivePerformance().simulationSuspended()) {
				actor.suspend();
			}
			this.startedActors.add(actor);
		}
	}

	public void removeActor(Actor actor) {
		synchronized (this.syncObject) {
			// this.startedActors.remove(actor);
			// actor.stop();
		}
	}

	public void handleStageChange(IStage stage) {
		synchronized (this.syncObject) {
			Play play = Play.getPlay();
			play.setActiveStage(stage);
			List<Actor> newActors = play.getActiveStage().getActors();
			Iterator<Actor> iter = newActors.iterator();
			while (iter.hasNext()) {
				Actor actor = iter.next();
				if (startedActors.contains(actor)) {
					iter.remove();
				}
			}
			for (Actor actor : newActors) {
				actor.start();
			}
			startedActors.addAll(newActors);
			play.getStagePanel().requestFocusInWindow();
		}
	}

}
