package simulation.listener;

import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import listener.TheaterMouseListener;
import model.Play;
import simulation.SimulationManager;
import theater.Component;
import theater.MouseInfo;
import theater.Stage;
import theater_intern.IComponent;
import theater_intern.IPerformance;
import theater_intern.IStage;
import util.Utils;
import view.StagePanel;

/**
 * Reagiert auf MouseEvents im Zustand running
 * 
 * @author Dietrich Boles, Uni Oldenburg
 * @version 1.0 (12.11.2008)
 * 
 */
public class SimulationMouseListener extends TheaterMouseListener {

	private List<Object> concernedObjects;
	private List<Object> movedObjects;
	private StagePanel stagePanel;
	private IStage iStage;

	public SimulationMouseListener() {
		this.movedObjects = new ArrayList<Object>();
	}

	public void mousePressed(final MouseEvent e) {
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {
					SimulationMouseListener.this.stagePanel = Play.getPlay()
							.getStagePanel();
					SimulationMouseListener.this.stagePanel
							.requestFocusInWindow();
					SimulationMouseListener.this.iStage = Play.getPlay()
							.getActiveStage();
					SimulationMouseListener.this.calcConcernedObjects(e);

					MouseInfo mouseInfo = new MouseInfo(e);
					mouseInfo.setColumn(SimulationMouseListener.this.stagePanel
							.getCellColumn(e.getX()));
					mouseInfo.setRow(SimulationMouseListener.this.stagePanel
							.getCellRow(e.getY()));
					for (Object obj : SimulationMouseListener.this.concernedObjects) {
						if (Utils.dynInstanceof(obj, IComponent.class)) {
							IComponent iComp = (IComponent) obj;
							Component comp = IComponent.getComp(iComp);
							java.awt.Rectangle rect = SimulationMouseListener.this.stagePanel
									.getStageRectangle(iComp);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX())
											- rect.x);
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY())
											- rect.y);
							mouseInfo.setSource(comp);
							simManager.newEventObject(new MousePressedObject(
									comp, mouseInfo));
						} else if (Utils.dynInstanceof(obj, IStage.class)) {
							IStage iStage = (IStage) obj;
							Stage stage = IStage.getStage(iStage);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX()));
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY()));
							mouseInfo.setSource(stage);
							simManager.newEventObject(new MousePressedObject(
									stage, mouseInfo));
						}
					}
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}

	public void mouseReleased(final MouseEvent e) {
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {
					SimulationMouseListener.this.stagePanel = Play.getPlay()
							.getStagePanel();
					SimulationMouseListener.this.stagePanel
							.requestFocusInWindow();
					SimulationMouseListener.this.iStage = Play.getPlay()
							.getActiveStage();
					SimulationMouseListener.this.calcConcernedObjects(e);

					MouseInfo mouseInfo = new MouseInfo(e);
					mouseInfo.setColumn(SimulationMouseListener.this.stagePanel
							.getCellColumn(e.getX()));
					mouseInfo.setRow(SimulationMouseListener.this.stagePanel
							.getCellRow(e.getY()));
					for (Object obj : SimulationMouseListener.this.concernedObjects) {
						if (Utils.dynInstanceof(obj, IComponent.class)) {
							IComponent iComp = (IComponent) obj;
							Component comp = IComponent.getComp(iComp);
							java.awt.Rectangle rect = SimulationMouseListener.this.stagePanel
									.getStageRectangle(iComp);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX())
											- rect.x);
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY())
											- rect.y);
							mouseInfo.setSource(comp);
							simManager.newEventObject(new MouseReleasedObject(
									comp, mouseInfo));
						} else if (Utils.dynInstanceof(obj, IStage.class)) {
							IStage iStage = (IStage) obj;
							Stage stage = IStage.getStage(iStage);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX()));
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY()));
							mouseInfo.setSource(stage);
							simManager.newEventObject(new MouseReleasedObject(
									stage, mouseInfo));
						}
					}
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}

	public void mouseClicked(final MouseEvent e) {
		// Anmerkung: kann prinzipiell falsch sein, wenn sich eine Komponente in
		// ihrer act- oder run-Methode bewegt; das Event wird generiert, wenn
		// sich beim Loslassen der Mauscursor ber der Komponente befindet
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {
					SimulationMouseListener.this.stagePanel = Play.getPlay()
							.getStagePanel();
					SimulationMouseListener.this.stagePanel
							.requestFocusInWindow();
					SimulationMouseListener.this.iStage = Play.getPlay()
							.getActiveStage();
					SimulationMouseListener.this.calcConcernedObjects(e);

					MouseInfo mouseInfo = new MouseInfo(e);
					mouseInfo.setColumn(SimulationMouseListener.this.stagePanel
							.getCellColumn(e.getX()));
					mouseInfo.setRow(SimulationMouseListener.this.stagePanel
							.getCellRow(e.getY()));
					for (Object obj : SimulationMouseListener.this.concernedObjects) {
						if (Utils.dynInstanceof(obj, IComponent.class)) {
							IComponent iComp = (IComponent) obj;
							Component comp = IComponent.getComp(iComp);
							java.awt.Rectangle rect = SimulationMouseListener.this.stagePanel
									.getStageRectangle(iComp);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX())
											- rect.x);
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY())
											- rect.y);
							mouseInfo.setSource(comp);
							simManager.newEventObject(new MouseClickedObject(
									comp, mouseInfo));
						} else if (Utils.dynInstanceof(obj, IStage.class)) {
							IStage iStage = (IStage) obj;
							Stage stage = IStage.getStage(iStage);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX()));
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY()));
							mouseInfo.setSource(stage);
							simManager.newEventObject(new MouseClickedObject(
									stage, mouseInfo));
						}
					}
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}

	public void mouseDragged(final MouseEvent e) {
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {
					SimulationMouseListener.this.stagePanel = Play.getPlay()
							.getStagePanel();
					SimulationMouseListener.this.stagePanel
							.requestFocusInWindow();
					SimulationMouseListener.this.iStage = Play.getPlay()
							.getActiveStage();
					SimulationMouseListener.this.calcConcernedObjects(e);

					MouseInfo mouseInfo = new MouseInfo(e);
					mouseInfo.setColumn(SimulationMouseListener.this.stagePanel
							.getCellColumn(e.getX()));
					mouseInfo.setRow(SimulationMouseListener.this.stagePanel
							.getCellRow(e.getY()));
					for (Object obj : SimulationMouseListener.this.concernedObjects) {
						if (Utils.dynInstanceof(obj, IComponent.class)) {
							IComponent iComp = (IComponent) obj;
							Component comp = IComponent.getComp(iComp);
							java.awt.Rectangle rect = SimulationMouseListener.this.stagePanel
									.getStageRectangle(iComp);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX())
											- rect.x);
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY())
											- rect.y);
							mouseInfo.setSource(comp);
							simManager.newEventObject(new MouseDraggedObject(
									comp, mouseInfo));
						} else if (Utils.dynInstanceof(obj, IStage.class)) {
							IStage iStage = (IStage) obj;
							Stage stage = IStage.getStage(iStage);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX()));
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY()));
							mouseInfo.setSource(stage);
							simManager.newEventObject(new MouseDraggedObject(
									stage, mouseInfo));
						}
					}
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}

	public void mouseEntered(MouseEvent e) {
		// siehe mouseMoved
	}

	public void mouseExited(MouseEvent e) {
		// siehe mouseMoved
	}

	public void mouseMoved(final MouseEvent e) {
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {
					SimulationMouseListener.this.stagePanel = Play.getPlay()
							.getStagePanel();
					// this.stagePanel.requestFocusInWindow();
					SimulationMouseListener.this.iStage = Play.getPlay()
							.getActiveStage();
					SimulationMouseListener.this.calcConcernedObjects(e);
					List<Object> nowMovedObjects = new ArrayList<Object>();

					MouseInfo mouseInfo = new MouseInfo(e);
					mouseInfo.setColumn(SimulationMouseListener.this.stagePanel
							.getCellColumn(e.getX()));
					mouseInfo.setRow(SimulationMouseListener.this.stagePanel
							.getCellRow(e.getY()));
					for (Object obj : SimulationMouseListener.this.concernedObjects) {
						if (Utils.dynInstanceof(obj, IComponent.class)) {
							IComponent iComp = (IComponent) obj;
							Component comp = IComponent.getComp(iComp);
							java.awt.Rectangle rect = SimulationMouseListener.this.stagePanel
									.getStageRectangle(iComp);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX())
											- rect.x);
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY())
											- rect.y);
							mouseInfo.setSource(comp);
							if (SimulationMouseListener.this.movedObjects
									.contains(comp)) {
								nowMovedObjects.add(comp);
								simManager.newEventObject(new MouseMovedObject(
										comp, mouseInfo));
							} else {
								nowMovedObjects.add(comp);
								simManager
										.newEventObject(new MouseEnteredObject(
												comp, mouseInfo));
							}
						} else if (Utils.dynInstanceof(obj, IStage.class)) {
							IStage iStage = (IStage) obj;
							Stage stage = IStage.getStage(iStage);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX()));
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY()));
							mouseInfo.setSource(stage);
							if (SimulationMouseListener.this.movedObjects
									.contains(stage)) {
								nowMovedObjects.add(stage);
								simManager.newEventObject(new MouseMovedObject(
										stage, mouseInfo));
							} else {
								nowMovedObjects.add(stage);
								simManager
										.newEventObject(new MouseEnteredObject(
												stage, mouseInfo));
							}
						}
					}

					// handle mouseExited
					// Anmerkung: kann bei sich bewegenden Akteure fehlschlagen
					// :-(
					IStage iStage = Play.getPlay().getActiveStage();
					List<IComponent> iComps = iStage
							.getComponentsInPaintOrder();
					for (int i = iComps.size() - 1; i >= 0; i--) {
						IComponent iComp = iComps.get(i);
						Component comp = IComponent.getComp(iComp);
						if (SimulationMouseListener.this.movedObjects
								.contains(comp)
								&& !nowMovedObjects.contains(comp)) {
							java.awt.Rectangle rect = SimulationMouseListener.this.stagePanel
									.getStageRectangle(iComp);
							mouseInfo
									.setX(SimulationMouseListener.this.stagePanel
											.getStageX(e.getX())
											- rect.x);
							mouseInfo
									.setY(SimulationMouseListener.this.stagePanel
											.getStageY(e.getY())
											- rect.y);
							mouseInfo.setSource(comp);
							simManager.newEventObject(new MouseExitedObject(
									comp, mouseInfo));
						}
					}

					Stage stage = IStage.getStage(iStage);
					if (SimulationMouseListener.this.movedObjects
							.contains(stage)
							&& !nowMovedObjects.contains(stage)) {
						mouseInfo.setX(SimulationMouseListener.this.stagePanel
								.getStageX(e.getX()));
						mouseInfo.setY(SimulationMouseListener.this.stagePanel
								.getStageY(e.getY()));
						mouseInfo.setSource(stage);
						simManager.newEventObject(new MouseExitedObject(stage,
								mouseInfo));
					}
					SimulationMouseListener.this.movedObjects = nowMovedObjects;
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}

	private void calcConcernedObjects(MouseEvent e) {
		this.concernedObjects = new ArrayList<Object>();
		int stageX = this.stagePanel.getStageX(e.getX());
		int stageY = this.stagePanel.getStageY(e.getY());
		List<IComponent> iComps = this.iStage.getComponentsInPaintOrder();
		for (int i = iComps.size() - 1; i >= 0; i--) {
			IComponent iComp = iComps.get(i);
			if (iComp.isHandlingMouseEvents()) {
				java.awt.Rectangle rect = this.stagePanel
						.getStageRectangle(iComp);
				if (rect.contains(new java.awt.Point(stageX, stageY))) {
					this.concernedObjects.add(iComp);
				}
			}
		}
		if (this.iStage.isHandlingMouseEvents()) {
			int cellSize = this.iStage.getCellSize();
			java.awt.Rectangle stageRect = new java.awt.Rectangle(0, 0,
					this.iStage.getNumberOfColumns() * cellSize, this.iStage
							.getNumberOfRows()
							* cellSize);
			if (stageRect.contains(new java.awt.Point(stageX, stageY))) {
				this.concernedObjects.add(this.iStage);
			}
		}

	}
}
