package simulation.listener;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.List;

import model.Play;
import simulation.SimulationManager;
import theater.Component;
import theater.KeyInfo;
import theater.Stage;
import theater_intern.IComponent;
import theater_intern.IPerformance;
import theater_intern.IStage;

public class TheaterKeyListener implements KeyListener {

	public TheaterKeyListener() {
		super();
	}

	public void keyPressed(final KeyEvent e) {
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {

					KeyInfo keyInfo = new KeyInfo(e);

					IStage iStage = Play.getPlay().getActiveStage();
					List<IComponent> iComps = iStage
							.getComponentsInPaintOrder();
					for (int i = iComps.size() - 1; i >= 0; i--) {
						IComponent iComp = iComps.get(i);
						Component comp = IComponent.getComp(iComp);
						simManager.newEventObject(new KeyPressedObject(comp,
								keyInfo));
					}
					Stage stage = IStage.getStage(iStage);
					simManager.newEventObject(new KeyPressedObject(stage,
							keyInfo));
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}

	public void keyReleased(final KeyEvent e) {
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {
					KeyInfo keyInfo = new KeyInfo(e);

					IStage iStage = Play.getPlay().getActiveStage();
					List<IComponent> iComps = iStage
							.getComponentsInPaintOrder();
					for (int i = iComps.size() - 1; i >= 0; i--) {
						IComponent iComp = iComps.get(i);
						Component comp = IComponent.getComp(iComp);
						simManager.newEventObject(new KeyReleasedObject(comp,
								keyInfo));
					}

					Stage stage = IStage.getStage(iStage);
					simManager.newEventObject(new KeyReleasedObject(stage,
							keyInfo));
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}

	public void keyTyped(final KeyEvent e) {
		new Thread() {
			public void run() {
				SimulationManager simManager = SimulationManager
						.getSimulationManager();
				IPerformance.lock();
				try {

					KeyInfo keyInfo = new KeyInfo(e);

					IStage iStage = Play.getPlay().getActiveStage();
					List<IComponent> iComps = iStage
							.getComponentsInPaintOrder();
					for (int i = iComps.size() - 1; i >= 0; i--) {
						IComponent iComp = iComps.get(i);
						Component comp = IComponent.getComp(iComp);
						simManager.newEventObject(new KeyTypedObject(comp,
								keyInfo));

					}
					Stage stage = IStage.getStage(iStage);
					simManager
							.newEventObject(new KeyTypedObject(stage, keyInfo));
				} finally {
					IPerformance.unlock();
				}
				simManager.processEventObjects();
			}
		}.start();
	}
}
